/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LANEINFO_H
#define DPC_IMPL_TYPE_LANEINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_array_linegeometry_4.h"
#include "dpc/impl_type_array_oddinfo_5.h"

namespace dpc {
struct LaneInfo {
    ::UInt8 lane_id{ 0U };
    ::Float max_speed{ 0.0F };
    ::Float min_speed{ 0.0F };
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };
    ::Float lane_merge_distance{ 0.0F };
    ::UInt8 lane_type{ 0U };
    ::dpc::array_LineGeometry_4 reference_line{};
    ::dpc::array_ODDInfo_5 odd_info{};

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(lane_id);
        fun(max_speed);
        fun(min_speed);
        fun(start_offset);
        fun(end_offset);
        fun(lane_merge_distance);
        fun(lane_type);
        fun(reference_line);
        fun(odd_info);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(lane_id);
        fun(max_speed);
        fun(min_speed);
        fun(start_offset);
        fun(end_offset);
        fun(lane_merge_distance);
        fun(lane_type);
        fun(reference_line);
        fun(odd_info);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("lane_id", lane_id);
        fun("max_speed", max_speed);
        fun("min_speed", min_speed);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("lane_merge_distance", lane_merge_distance);
        fun("lane_type", lane_type);
        fun("reference_line", reference_line);
        fun("odd_info", odd_info);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("lane_id", lane_id);
        fun("max_speed", max_speed);
        fun("min_speed", min_speed);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("lane_merge_distance", lane_merge_distance);
        fun("lane_type", lane_type);
        fun("reference_line", reference_line);
        fun("odd_info", odd_info);
    }

    friend bool operator==(const ::dpc::LaneInfo& lhs, const ::dpc::LaneInfo& rhs) noexcept
    {
        return (lhs.lane_id == rhs.lane_id) && (fabs(static_cast<double>(lhs.max_speed - rhs.max_speed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.min_speed - rhs.min_speed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.lane_merge_distance - rhs.lane_merge_distance)) < DBL_EPSILON) && (lhs.lane_type == rhs.lane_type) && (lhs.reference_line == rhs.reference_line) && (lhs.odd_info == rhs.odd_info);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LANEINFO_H
